@@ -1,33 +1,18 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "config.h"
 #include "PlayerCommands.hxx"
 #include "Request.hxx"
 #include "queue/Playlist.hxx"
 #include "PlaylistPrint.hxx"
+#include "ConsumeMode.hxx"
 #include "SingleMode.hxx"
 #include "client/Client.hxx"
 #include "client/Response.hxx"
 #include "Partition.hxx"
 #include "Instance.hxx"
-#include "IdleFlags.hxx"
+#include "protocol/IdleFlags.hxx"
 #include "lib/fmt/AudioFormatFormatter.hxx"
 #include "util/StringBuffer.hxx"
 #include "util/ScopeExit.hxx"
@@ -59,6 +44,7 @@
 #define COMMAND_STATUS_MIXRAMPDELAY	"mixrampdelay"
 #define COMMAND_STATUS_AUDIO		"audio"
 #define COMMAND_STATUS_UPDATING_DB	"updating_db"
+#define COMMAND_STATUS_LOADED_PLAYLIST  "lastloadedplaylist"
 
 CommandResult
 handle_play(Client &client, Request args, [[maybe_unused]] Response &r)
@@ -143,16 +129,18 @@ handle_status(Client &client, [[maybe_unused]] Request args, Response &r)
 	      COMMAND_STATUS_PLAYLIST ": {}\n"
 	      COMMAND_STATUS_PLAYLIST_LENGTH ": {}\n"
 	      COMMAND_STATUS_MIXRAMPDB ": {}\n"
-	      COMMAND_STATUS_STATE ": {}\n",
+	      COMMAND_STATUS_STATE ": {}\n"
+	      COMMAND_STATUS_LOADED_PLAYLIST ": {}\n",
 	      (unsigned)playlist.GetRepeat(),
 	      (unsigned)playlist.GetRandom(),
 	      SingleToString(playlist.GetSingle()),
-	      (unsigned)playlist.GetConsume(),
+	      ConsumeToString(playlist.GetConsume()),
 	      partition.name.c_str(),
 	      playlist.GetVersion(),
 	      playlist.GetLength(),
 	      pc.GetMixRampDb(),
-	      state);
+	      state,
+	      playlist.GetLastLoadedPlaylist());
 
 	if (pc.GetCrossFade() > FloatDuration::zero())
 		r.Fmt(COMMAND_STATUS_CROSSFADE ": {}\n",
@@ -261,8 +249,8 @@ handle_single(Client &client, Request args, [[maybe_unused]] Response &r)
 CommandResult
 handle_consume(Client &client, Request args, [[maybe_unused]] Response &r)
 {
-	bool status = args.ParseBool(0);
-	client.GetPartition().SetConsume(status);
+	auto new_mode = ConsumeFromString(args.front());
+	client.GetPartition().SetConsume(new_mode);
 	return CommandResult::OK;
 }
 
