@@ -1,31 +1,14 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "FileInputPlugin.hxx"
 #include "../InputStream.hxx"
 #include "fs/Path.hxx"
 #include "fs/FileInfo.hxx"
-#include "fs/io/FileReader.hxx"
+#include "lib/fmt/PathFormatter.hxx"
+#include "lib/fmt/RuntimeError.hxx"
+#include "io/FileReader.hxx"
 #include "io/FileDescriptor.hxx"
-#include "util/RuntimeError.hxx"
-
-#include <cinttypes> // for PRIu64 (PRIoffset)
 
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -50,7 +33,7 @@ public:
 	}
 
 	size_t Read(std::unique_lock<Mutex> &lock,
-		    void *ptr, size_t size) override;
+		    std::span<std::byte> dest) override;
 	void Seek(std::unique_lock<Mutex> &lock,
 		  offset_type offset) override;
 };
@@ -63,8 +46,7 @@ OpenFileInputStream(Path path, Mutex &mutex)
 	const FileInfo info = reader.GetFileInfo();
 
 	if (!info.IsRegular())
-		throw FormatRuntimeError("Not a regular file: %s",
-					 path.c_str());
+		throw FmtRuntimeError("Not a regular file: {}", path);
 
 #ifdef POSIX_FADV_SEQUENTIAL
 	posix_fadvise(reader.GetFD().Get(), (off_t)0, info.GetSize(),
@@ -89,20 +71,19 @@ FileInputStream::Seek(std::unique_lock<Mutex> &,
 }
 
 size_t
-FileInputStream::Read(std::unique_lock<Mutex> &,
-		      void *ptr, size_t read_size)
+FileInputStream::Read(std::unique_lock<Mutex> &, std::span<std::byte> dest)
 {
 	size_t nbytes;
 
 	{
 		const ScopeUnlock unlock(mutex);
-		nbytes = reader.Read(ptr, read_size);
+		nbytes = reader.Read(dest);
 	}
 
 	if (nbytes == 0 && !IsEOF())
-		throw FormatRuntimeError("Unexpected end of file %s"
-					 " at %" PRIoffset " of %" PRIoffset,
-					 GetURI(), GetOffset(), GetSize());
+		throw FmtRuntimeError("Unexpected end of file {}"
+				      " at {} of {}",
+				      GetURI(), GetOffset(), GetSize());
 
 	offset += nbytes;
 	return nbytes;
