@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "Connection.hxx"
 #include "Error.hxx"
@@ -23,7 +7,6 @@
 #include "Callback.hxx"
 #include "event/Loop.hxx"
 #include "net/SocketDescriptor.hxx"
-#include "util/RuntimeError.hxx"
 
 extern "C" {
 #include <nfsc/libnfs.h>
@@ -37,6 +20,8 @@ extern "C" {
 #include <poll.h> /* for POLLIN, POLLOUT */
 #endif
 
+#include <string.h> // for strstr()
+
 static constexpr Event::Duration NFS_MOUNT_TIMEOUT =
 	std::chrono::minutes(1);
 
@@ -48,8 +33,7 @@ NfsConnection::CancellableCallback::Stat(nfs_context *ctx,
 
 	int result = nfs_stat64_async(ctx, path, Callback, this);
 	if (result < 0)
-		throw FormatRuntimeError("nfs_stat64_async() failed: %s",
-					 nfs_get_error(ctx));
+		throw NfsClientError(ctx, "nfs_stat64_async() failed");
 }
 
 inline void
@@ -60,8 +44,7 @@ NfsConnection::CancellableCallback::Lstat(nfs_context *ctx,
 
 	int result = nfs_lstat64_async(ctx, path, Callback, this);
 	if (result < 0)
-		throw FormatRuntimeError("nfs_lstat64_async() failed: %s",
-					 nfs_get_error(ctx));
+		throw NfsClientError(ctx, "nfs_lstat64_async() failed");
 }
 
 inline void
@@ -72,8 +55,7 @@ NfsConnection::CancellableCallback::OpenDirectory(nfs_context *ctx,
 
 	int result = nfs_opendir_async(ctx, path, Callback, this);
 	if (result < 0)
-		throw FormatRuntimeError("nfs_opendir_async() failed: %s",
-					 nfs_get_error(ctx));
+		throw NfsClientError(ctx, "nfs_opendir_async() failed");
 }
 
 inline void
@@ -85,8 +67,7 @@ NfsConnection::CancellableCallback::Open(nfs_context *ctx,
 	int result = nfs_open_async(ctx, path, flags,
 				    Callback, this);
 	if (result < 0)
-		throw FormatRuntimeError("nfs_open_async() failed: %s",
-					 nfs_get_error(ctx));
+		throw NfsClientError(ctx, "nfs_open_async() failed");
 }
 
 inline void
@@ -95,33 +76,48 @@ NfsConnection::CancellableCallback::Stat(nfs_context *ctx,
 {
 	assert(connection.GetEventLoop().IsInside());
 
-	int result = nfs_fstat_async(ctx, fh, Callback, this);
+	int result = nfs_fstat64_async(ctx, fh, Callback, this);
 	if (result < 0)
-		throw FormatRuntimeError("nfs_fstat_async() failed: %s",
-					 nfs_get_error(ctx));
+		throw NfsClientError(ctx, "nfs_fstat_async() failed");
 }
 
 inline void
 NfsConnection::CancellableCallback::Read(nfs_context *ctx, struct nfsfh *fh,
-					 uint64_t offset, size_t size)
+					 uint64_t offset,
+#ifdef LIBNFS_API_2
+					 std::span<std::byte> dest
+#else
+					 std::size_t size
+#endif
+	)
 {
 	assert(connection.GetEventLoop().IsInside());
 
-	int result = nfs_pread_async(ctx, fh, offset, size, Callback, this);
+	int result = nfs_pread_async(ctx, fh,
+#ifdef LIBNFS_API_2
+				     dest.data(), dest.size(),
+#endif
+				     offset,
+#ifndef LIBNFS_API_2
+				     size,
+#endif
+				     Callback, this);
 	if (result < 0)
-		throw FormatRuntimeError("nfs_pread_async() failed: %s",
-					 nfs_get_error(ctx));
+		throw NfsClientError(ctx, "nfs_pread_async() failed");
 }
 
 inline void
-NfsConnection::CancellableCallback::CancelAndScheduleClose(struct nfsfh *fh) noexcept
+NfsConnection::CancellableCallback::CancelAndScheduleClose(struct nfsfh *fh,
+							   DisposablePointer &&_dispose_value) noexcept
 {
 	assert(connection.GetEventLoop().IsInside());
 	assert(!open);
 	assert(close_fh == nullptr);
-	assert(fh != nullptr);
+	assert(!dispose_value);
 
 	close_fh = fh;
+	dispose_value = std::move(_dispose_value);
+
 	Cancel();
 }
 
@@ -136,11 +132,27 @@ NfsConnection::CancellableCallback::PrepareDestroyContext() noexcept
 	}
 }
 
+/**
+ * Determine whether this is a fatal error and we need to close the
+ * connection and reopen a new one.
+ */
+static constexpr bool
+IsFatalNfsConnectionError(int err, const char *msg) noexcept
+{
+	/* nfsstat3_to_errno() translates NFS4ERR_EXPIRED to ERANGE,
+	   so unfortunately we need to search the error message */
+	return err == -ERANGE && msg != nullptr &&
+	       strstr(msg, "NFS4ERR_EXPIRED") != nullptr;
+}
+
 inline void
 NfsConnection::CancellableCallback::Callback(int err, void *data) noexcept
 {
 	assert(connection.GetEventLoop().IsInside());
 
+	auto &_connection = connection;
+	const bool is_fatal = IsFatalNfsConnectionError(err, (const char *)data);
+
 	if (!IsCancelled()) {
 		assert(close_fh == nullptr);
 
@@ -150,7 +162,7 @@ NfsConnection::CancellableCallback::Callback(int err, void *data) noexcept
 
 		if (err >= 0)
 			cb.OnNfsCallback((unsigned)err, data);
-		else
+		else 
 			cb.OnNfsError(std::make_exception_ptr(NfsClientError(-err, (const char *)data)));
 	} else {
 		if (open) {
@@ -168,6 +180,11 @@ NfsConnection::CancellableCallback::Callback(int err, void *data) noexcept
 
 		connection.callbacks.Remove(*this);
 	}
+
+	if (is_fatal)
+		/* this was a fatal connection error; we need to teat
+		   down the NFS connection */
+		_connection.BroadcastError(std::make_exception_ptr(NfsClientError(-err, (const char *)data)));
 }
 
 void
@@ -196,6 +213,18 @@ events_to_libnfs(unsigned i) noexcept
 		((i & SocketEvent::ERROR) ? POLLERR : 0);
 }
 
+NfsConnection::NfsConnection(EventLoop &_loop,
+			     nfs_context *_context,
+			     std::string_view _server,
+			     std::string_view _export_name)
+	:socket_event(_loop, BIND_THIS_METHOD(OnSocketReady)),
+	 defer_new_lease(_loop, BIND_THIS_METHOD(RunDeferred)),
+	 mount_timeout_event(_loop, BIND_THIS_METHOD(OnMountTimeout)),
+	 server(_server), export_name(_export_name),
+	 context(_context)
+{
+}
+
 NfsConnection::~NfsConnection() noexcept
 {
 	assert(GetEventLoop().IsInside());
@@ -203,17 +232,24 @@ NfsConnection::~NfsConnection() noexcept
 	assert(active_leases.empty());
 	assert(callbacks.IsEmpty());
 	assert(deferred_close.empty());
+	assert(context != nullptr);
 
-	if (context != nullptr)
-		DestroyContext();
+	socket_event.ReleaseSocket();
+
+	/* cancel the timer before calling nfs_destroy_context() so
+	   MountCallback() knows the mount call is being canceled;
+	   checking status==-EINTR would be ambiguous */
+	mount_timeout_event.Cancel();
+
+	nfs_destroy_context(context);
 }
 
 void
 NfsConnection::AddLease(NfsLease &lease) noexcept
 {
-	assert(GetEventLoop().IsInside());
+	assert(!GetEventLoop().IsAlive() || GetEventLoop().IsInside());
 
-	new_leases.push_back(&lease);
+	new_leases.push_back(lease);
 
 	defer_new_lease.Schedule();
 }
@@ -221,10 +257,9 @@ NfsConnection::AddLease(NfsLease &lease) noexcept
 void
 NfsConnection::RemoveLease(NfsLease &lease) noexcept
 {
-	assert(GetEventLoop().IsInside());
+	assert(!GetEventLoop().IsAlive() || GetEventLoop().IsInside());
 
-	new_leases.remove(&lease);
-	active_leases.remove(&lease);
+	lease.unlink();
 }
 
 void
@@ -329,7 +364,12 @@ NfsConnection::Stat(struct nfsfh *fh, NfsCallback &callback)
 }
 
 void
-NfsConnection::Read(struct nfsfh *fh, uint64_t offset, size_t size,
+NfsConnection::Read(struct nfsfh *fh, uint64_t offset,
+#ifdef LIBNFS_API_2
+		    std::span<std::byte> dest,
+#else
+		    std::size_t size,
+#endif
 		    NfsCallback &callback)
 {
 	assert(GetEventLoop().IsInside());
@@ -337,7 +377,13 @@ NfsConnection::Read(struct nfsfh *fh, uint64_t offset, size_t size,
 
 	auto &c = callbacks.Add(callback, *this, false);
 	try {
-		c.Read(context, fh, offset, size);
+		c.Read(context, fh, offset,
+#ifdef LIBNFS_API_2
+		       dest
+#else
+		       size
+#endif
+			);
 	} catch (...) {
 		callbacks.Remove(c);
 		throw;
@@ -347,9 +393,12 @@ NfsConnection::Read(struct nfsfh *fh, uint64_t offset, size_t size,
 }
 
 void
-NfsConnection::Cancel(NfsCallback &callback) noexcept
+NfsConnection::Cancel(NfsCallback &callback,
+		      struct nfsfh *fh,
+		      DisposablePointer dispose_value) noexcept
 {
-	callbacks.Cancel(callback);
+	CancellableCallback &cancel = callbacks.Get(callback);
+	cancel.CancelAndScheduleClose(fh, std::move(dispose_value));
 }
 
 static void
@@ -361,7 +410,6 @@ inline void
 NfsConnection::InternalClose(struct nfsfh *fh) noexcept
 {
 	assert(GetEventLoop().IsInside());
-	assert(context != nullptr);
 	assert(fh != nullptr);
 
 	nfs_close_async(context, fh, DummyCallback, nullptr);
@@ -377,26 +425,15 @@ NfsConnection::Close(struct nfsfh *fh) noexcept
 }
 
 void
-NfsConnection::CancelAndClose(struct nfsfh *fh, NfsCallback &callback) noexcept
-{
-	CancellableCallback &cancel = callbacks.Get(callback);
-	cancel.CancelAndScheduleClose(fh);
-}
-
-void
-NfsConnection::DestroyContext() noexcept
+NfsConnection::PrepareDestroyContext() noexcept
 {
 	assert(GetEventLoop().IsInside());
-	assert(context != nullptr);
 
-#ifndef NDEBUG
-	assert(!in_destroy);
-	in_destroy = true;
-#endif
-
-	if (!mount_finished) {
+	if (mount_state == MountState::WAITING) {
 		assert(mount_timeout_event.IsPending());
 		mount_timeout_event.Cancel();
+	} else {
+		assert(!mount_timeout_event.IsPending());
 	}
 
 	/* cancel pending DeferEvent that was scheduled to notify
@@ -408,9 +445,6 @@ NfsConnection::DestroyContext() noexcept
 	callbacks.ForEach([](CancellableCallback &c){
 			c.PrepareDestroyContext();
 		});
-
-	nfs_destroy_context(context);
-	context = nullptr;
 }
 
 inline void
@@ -419,7 +453,6 @@ NfsConnection::DeferClose(struct nfsfh *fh) noexcept
 	assert(GetEventLoop().IsInside());
 	assert(in_event);
 	assert(in_service);
-	assert(context != nullptr);
 	assert(fh != nullptr);
 
 	deferred_close.push_front(fh);
@@ -429,7 +462,6 @@ void
 NfsConnection::ScheduleSocket() noexcept
 {
 	assert(GetEventLoop().IsInside());
-	assert(context != nullptr);
 
 	const int which_events = nfs_which_events(context);
 
@@ -457,7 +489,6 @@ inline int
 NfsConnection::Service(unsigned flags) noexcept
 {
 	assert(GetEventLoop().IsInside());
-	assert(context != nullptr);
 
 #ifndef NDEBUG
 	assert(!in_event);
@@ -470,7 +501,6 @@ NfsConnection::Service(unsigned flags) noexcept
 	int result = nfs_service(context, events_to_libnfs(flags));
 
 #ifndef NDEBUG
-	assert(context != nullptr);
 	assert(in_service);
 	in_service = false;
 #endif
@@ -483,9 +513,11 @@ NfsConnection::OnSocketReady(unsigned flags) noexcept
 {
 	assert(GetEventLoop().IsInside());
 	assert(deferred_close.empty());
+	assert(mount_state >= MountState::WAITING);
+	assert(!postponed_mount_error);
 
-	const bool was_mounted = mount_finished;
-	if (!mount_finished || (flags & SocketEvent::HANGUP) != 0)
+	const bool was_mounted = mount_state >= MountState::FINISHED;
+	if (!was_mounted || (flags & SocketEvent::HANGUP) != 0)
 		/* until the mount is finished, the NFS client may use
 		   various sockets, therefore we unregister and
 		   re-register it each time */
@@ -502,61 +534,65 @@ NfsConnection::OnSocketReady(unsigned flags) noexcept
 		deferred_close.pop_front();
 	}
 
-	if (!was_mounted && mount_finished) {
-		if (postponed_mount_error) {
-			DestroyContext();
-			BroadcastMountError(std::move(postponed_mount_error));
-		} else if (result == 0)
-			BroadcastMountSuccess();
-	} else if (result < 0) {
-		/* the connection has failed */
+	try {
+		if (postponed_mount_error)
+			std::rethrow_exception(postponed_mount_error);
 
-		auto e = FormatRuntimeError("NFS connection has failed: %s",
-					    nfs_get_error(context));
-		BroadcastError(std::make_exception_ptr(e));
+		if (result < 0)
+			throw NfsClientError(context, "NFS connection has failed");
 
-		DestroyContext();
-	} else if (nfs_get_fd(context) < 0) {
-		/* this happens when rpc_reconnect_requeue() is called
-		   after the connection broke, but autoreconnect was
-		   disabled - nfs_service() returns 0 */
-
-		const char *msg = nfs_get_error(context);
-		if (msg == nullptr)
-			msg = "<unknown>";
-		auto e = FormatRuntimeError("NFS socket disappeared: %s", msg);
-
-		BroadcastError(std::make_exception_ptr(e));
-
-		DestroyContext();
+		if (nfs_get_fd(context) < 0)
+			/* this happens when rpc_reconnect_requeue()
+			   is called after the connection broke, but
+			   autoreconnect was disabled - nfs_service()
+			   returns 0 */
+			throw NfsClientError(context, "NFS socket disappeared");
+	} catch (...) {
+		PrepareDestroyContext();
+		if (was_mounted)
+			BroadcastError(std::current_exception());
+		else
+			BroadcastMountError(std::current_exception());
+		return;
 	}
 
-	assert(context == nullptr || nfs_get_fd(context) >= 0);
+	assert(nfs_get_fd(context) >= 0);
+
+	if (!was_mounted && mount_state >= MountState::FINISHED)
+		BroadcastMountSuccess();
 
 #ifndef NDEBUG
 	assert(in_event);
 	in_event = false;
 #endif
 
-	if (context != nullptr)
-		ScheduleSocket();
+	ScheduleSocket();
 }
 
 inline void
 NfsConnection::MountCallback(int status, [[maybe_unused]] nfs_context *nfs,
-			     [[maybe_unused]] void *data) noexcept
+			     void *data) noexcept
 {
 	assert(GetEventLoop().IsInside());
 	assert(context == nfs);
 
-	mount_finished = true;
+	if (!mount_timeout_event.IsPending()) {
+		/* called by nfs_destroy_context() while destructing
+		   this NfsConnection instance */
+		assert(status == -EINTR);
+		assert(mount_state == MountState::FINISHED);
+		return;
+	}
+
+	assert(mount_state == MountState::WAITING);
+
+	mount_state = MountState::FINISHED;
 
-	assert(mount_timeout_event.IsPending() || in_destroy);
 	mount_timeout_event.Cancel();
 
 	if (status < 0) {
-		auto e = FormatRuntimeError("nfs_mount_async() failed: %s",
-					    nfs_get_error(context));
+		auto e = NfsClientError(status, context, data,
+					"nfs_mount_async() failed");
 		postponed_mount_error = std::make_exception_ptr(e);
 		return;
 	}
@@ -575,30 +611,16 @@ inline void
 NfsConnection::MountInternal()
 {
 	assert(GetEventLoop().IsInside());
-	assert(context == nullptr);
+	assert(mount_state == MountState::INITIAL);
 
-	context = nfs_init_context();
-	if (context == nullptr)
-		throw std::runtime_error("nfs_init_context() failed");
-
-	postponed_mount_error = std::exception_ptr();
-	mount_finished = false;
+	mount_state = MountState::WAITING;
 
 	mount_timeout_event.Schedule(NFS_MOUNT_TIMEOUT);
 
-#ifndef NDEBUG
-	in_service = false;
-	in_event = false;
-	in_destroy = false;
-#endif
-
 	if (nfs_mount_async(context, server.c_str(), export_name.c_str(),
 			    MountCallback, this) != 0) {
-		auto e = FormatRuntimeError("nfs_mount_async() failed: %s",
-					    nfs_get_error(context));
-		nfs_destroy_context(context);
-		context = nullptr;
-		throw e;
+		mount_state = MountState::FINISHED;
+		throw NfsClientError(context, "nfs_mount_async() failed");
 	}
 
 	ScheduleSocket();
@@ -608,38 +630,40 @@ void
 NfsConnection::BroadcastMountSuccess() noexcept
 {
 	assert(GetEventLoop().IsInside());
+	assert(mount_state == MountState::FINISHED);
 
-	while (!new_leases.empty()) {
-		auto i = new_leases.begin();
-		active_leases.splice(active_leases.end(), new_leases, i);
-		(*i)->OnNfsConnectionReady();
-	}
+	new_leases.clear_and_dispose([this](auto *lease){
+#ifdef __clang__
+		/* suppress bogus clang-18 -Wunused-lambda-capture warning */
+		(void)this;
+#endif
+
+		active_leases.push_back(*lease);
+		lease->OnNfsConnectionReady();
+	});
 }
 
 void
-NfsConnection::BroadcastMountError(std::exception_ptr &&e) noexcept
+NfsConnection::BroadcastMountError(std::exception_ptr e) noexcept
 {
 	assert(GetEventLoop().IsInside());
+	assert(mount_state == MountState::FINISHED);
 
-	while (!new_leases.empty()) {
-		auto l = new_leases.front();
-		new_leases.pop_front();
+	new_leases.clear_and_dispose([&e](auto *l){
 		l->OnNfsConnectionFailed(e);
-	}
+	});
 
 	OnNfsConnectionError(std::move(e));
 }
 
 void
-NfsConnection::BroadcastError(std::exception_ptr &&e) noexcept
+NfsConnection::BroadcastError(std::exception_ptr e) noexcept
 {
 	assert(GetEventLoop().IsInside());
 
-	while (!active_leases.empty()) {
-		auto l = active_leases.front();
-		active_leases.pop_front();
+	active_leases.clear_and_dispose([&e](auto *l){
 		l->OnNfsConnectionDisconnected(e);
-	}
+	});
 
 	BroadcastMountError(std::move(e));
 }
@@ -648,10 +672,10 @@ void
 NfsConnection::OnMountTimeout() noexcept
 {
 	assert(GetEventLoop().IsInside());
-	assert(!mount_finished);
+	assert(mount_state == MountState::WAITING);
 
-	mount_finished = true;
-	DestroyContext();
+	mount_state = MountState::FINISHED;
+	PrepareDestroyContext();
 
 	BroadcastMountError(std::make_exception_ptr(std::runtime_error("Mount timeout")));
 }
@@ -661,7 +685,7 @@ NfsConnection::RunDeferred() noexcept
 {
 	assert(GetEventLoop().IsInside());
 
-	if (context == nullptr) {
+	if (mount_state == MountState::INITIAL) {
 		try {
 			MountInternal();
 		} catch (...) {
@@ -670,6 +694,6 @@ NfsConnection::RunDeferred() noexcept
 		}
 	}
 
-	if (mount_finished)
+	if (mount_state >= MountState::FINISHED)
 		BroadcastMountSuccess();
 }
