@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2020 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "GlueResampler.hxx"
 #include "ConfiguredResampler.hxx"
@@ -71,18 +55,16 @@ GluePcmResampler::Reset() noexcept
 	resampler->Reset();
 }
 
-ConstBuffer<void>
-GluePcmResampler::Resample(ConstBuffer<void> src)
+std::span<const std::byte>
+GluePcmResampler::Resample(std::span<const std::byte> src)
 {
-	assert(!src.IsNull());
-
 	if (requested_sample_format != src_sample_format)
 		src = format_converter.Convert(src);
 
 	return resampler->Resample(src);
 }
 
-ConstBuffer<void>
+std::span<const std::byte>
 GluePcmResampler::Flush()
 {
 	return resampler->Flush();
