@@ -1,26 +1,10 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "SoxrResampler.hxx"
 #include "AudioFormat.hxx"
 #include "config/Block.hxx"
-#include "util/RuntimeError.hxx"
+#include "lib/fmt/RuntimeError.hxx"
 #include "util/Domain.hxx"
 #include "Log.hxx"
 
@@ -64,8 +48,7 @@ static constexpr struct {
 	{ SOXR_INVALID_RECIPE, nullptr }
 };
 
-gcc_const
-static const char *
+static constexpr const char *
 soxr_quality_name(unsigned long recipe) noexcept
 {
 	for (const auto *i = soxr_quality_table;; ++i) {
@@ -76,7 +59,7 @@ soxr_quality_name(unsigned long recipe) noexcept
 	}
 }
 
-gcc_pure
+[[gnu::pure]]
 static unsigned long
 soxr_parse_quality(const char *quality) noexcept
 {
@@ -100,18 +83,17 @@ SoxrParsePrecision(unsigned value) {
 	case 32:
 		break;
 	default:
-		throw FormatInvalidArgument(
-			"soxr converter invalid precision : %d [16|20|24|28|32]", value);
+		throw FmtInvalidArgument("soxr converter invalid precision: {} [16|20|24|28|32]",
+					 value);
 	}
 	return value;
 }
 
 static double
 SoxrParsePhaseResponse(unsigned value) {
-	if (value > 100) {
-		throw FormatInvalidArgument(
-			"soxr converter invalid phase_respons : %d (0-100)", value);
-	}
+	if (value > 100)
+		throw FmtInvalidArgument("soxr converter invalid phase_respons : {} (0-100)",
+					 value);
 
 	return double(value);
 }
@@ -120,15 +102,13 @@ static double
 SoxrParsePassbandEnd(const char *svalue) {
 	char *endptr;
 	double value = strtod(svalue, &endptr);
-	if (svalue == endptr || *endptr != 0) {
-		throw FormatInvalidArgument(
-			"soxr converter passband_end value not a number: %s", svalue);
-	}
+	if (svalue == endptr || *endptr != 0)
+		throw FmtInvalidArgument("soxr converter passband_end value not a number: {}",
+					 svalue);
 
-	if (value < 1 || value > 100) {
-		throw FormatInvalidArgument(
-			"soxr converter invalid passband_end : %s (1-100%%)", svalue);
-	}
+	if (value < 1 || value > 100)
+		throw FmtInvalidArgument("soxr converter invalid passband_end: {} (1-100%)",
+					 svalue);
 
 	return value / 100.0;
 }
@@ -137,15 +117,13 @@ static double
 SoxrParseStopbandBegin(const char *svalue) {
 	char *endptr;
 	double value = strtod(svalue, &endptr);
-	if (svalue == endptr || *endptr != 0) {
-		throw FormatInvalidArgument(
-			"soxr converter stopband_begin value not a number: %s", svalue);
-	}
+	if (svalue == endptr || *endptr != 0)
+		throw FmtInvalidArgument("soxr converter stopband_begin value not a number: {}",
+					 svalue);
 
-	if (value < 100 || value > 199) {
-		throw FormatInvalidArgument(
-			"soxr converter invalid stopband_begin : %s (100-150%%)", svalue);
-	}
+	if (value < 100 || value > 199)
+		throw FmtInvalidArgument("soxr converter invalid stopband_begin: {} (100-150%)",
+					 svalue);
 
 	return value / 100.0;
 }
@@ -155,14 +133,13 @@ SoxrParseAttenuation(const char *svalue) {
 	char *endptr;
 	double value = strtod(svalue, &endptr);
 	if (svalue == endptr || *endptr != 0) {
-		throw FormatInvalidArgument(
-			"soxr converter attenuation value not a number: %s", svalue);
+		throw FmtInvalidArgument("soxr converter attenuation value not a number: {}",
+					 svalue);
 	}
 
-	if (value < 0 || value > 30) {
-		throw FormatInvalidArgument(
-			"soxr converter invalid attenuation : %s (0-30dB))", svalue);
-	}
+	if (value < 0 || value > 30)
+		throw FmtInvalidArgument("soxr converter invalid attenuation: {} (0-30dB))",
+					 svalue);
 
 	return 1 / std::pow(10, value / 10.0);
 }
@@ -176,8 +153,8 @@ pcm_resample_soxr_global_init(const ConfigBlock &block)
 
 	if (recipe == SOXR_INVALID_RECIPE) {
 		assert(quality_string != nullptr);
-		throw FormatRuntimeError("unknown quality setting '%s' in line %d",
-					 quality_string, block.line);
+		throw FmtRuntimeError("unknown quality setting {:?} in line {}",
+				      quality_string, block.line);
 	} else if (recipe == SOXR_CUSTOM_RECIPE) {
 		// used to preset possible internal flags, like SOXR_RESET_ON_CLEAR
 		soxr_quality = soxr_quality_spec(SOXR_DEFAULT_RECIPE, 0);
@@ -200,7 +177,7 @@ pcm_resample_soxr_global_init(const ConfigBlock &block)
 		soxr_quality = soxr_quality_spec(recipe, 0);
 	}
 
-	FmtDebug(soxr_domain, "soxr converter '{}'",
+	FmtDebug(soxr_domain, "soxr converter {:?}",
 		 soxr_quality_name(recipe));
 
 	const unsigned n_threads = block.GetBlockValue("threads", 1);
@@ -222,10 +199,10 @@ SoxrPcmResampler::Open(AudioFormat &af, unsigned new_sample_rate)
 			   af.channels, &e,
 			   p_soxr_io, &soxr_quality, &soxr_runtime);
 	if (soxr == nullptr)
-		throw FormatRuntimeError("soxr initialization has failed: %s",
-					 e);
+		throw FmtRuntimeError("soxr initialization has failed: {}",
+				      e);
 
-	FmtDebug(soxr_domain, "soxr engine '{}'", soxr_engine(soxr));
+	FmtDebug(soxr_domain, "soxr engine {:?}", soxr_engine(soxr));
 	if (soxr_use_custom_recipe)
 		FmtDebug(soxr_domain,
 			 "soxr precision={:0.0f}, phase_response={:0.2f}, "
@@ -262,18 +239,16 @@ SoxrPcmResampler::Close() noexcept
 void
 SoxrPcmResampler::Reset() noexcept
 {
-#if SOXR_THIS_VERSION >= SOXR_VERSION(0,1,2)
 	soxr_clear(soxr);
-#endif
 }
 
-ConstBuffer<void>
-SoxrPcmResampler::Resample(ConstBuffer<void> src)
+std::span<const std::byte>
+SoxrPcmResampler::Resample(std::span<const std::byte> src)
 {
 	const size_t frame_size = channels * sizeof(float);
-	assert(src.size % frame_size == 0);
+	assert(src.size() % frame_size == 0);
 
-	const size_t n_frames = src.size / frame_size;
+	const size_t n_frames = src.size() / frame_size;
 
 	/* always round up: worst case output buffer size */
 	const size_t o_frames = size_t(n_frames * ratio) + 1;
@@ -281,15 +256,15 @@ SoxrPcmResampler::Resample(ConstBuffer<void> src)
 	auto *output_buffer = (float *)buffer.Get(o_frames * frame_size);
 
 	size_t i_done, o_done;
-	soxr_error_t e = soxr_process(soxr, src.data, n_frames, &i_done,
+	soxr_error_t e = soxr_process(soxr, src.data(), n_frames, &i_done,
 				      output_buffer, o_frames, &o_done);
 	if (e != nullptr)
-		throw FormatRuntimeError("soxr error: %s", e);
+		throw FmtRuntimeError("soxr error: {}", e);
 
-	return { output_buffer, o_done * frame_size };
+	return { (const std::byte *)output_buffer, o_done * frame_size };
 }
 
-ConstBuffer<void>
+std::span<const std::byte>
 SoxrPcmResampler::Flush()
 {
 	const size_t frame_size = channels * sizeof(float);
@@ -301,11 +276,11 @@ SoxrPcmResampler::Flush()
 	soxr_error_t e = soxr_process(soxr, nullptr, 0, nullptr,
 				      output_buffer, o_frames, &o_done);
 	if (e != nullptr)
-		throw FormatRuntimeError("soxr error: %s", e);
+		throw FmtRuntimeError("soxr error: {}", e);
 
 	if (o_done == 0)
 		/* flush complete */
 		output_buffer = nullptr;
 
-	return { output_buffer, o_done * frame_size };
+	return { (const std::byte *)output_buffer, o_done * frame_size };
 }
