@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "XspfPlaylistPlugin.hxx"
 #include "../PlaylistPlugin.hxx"
@@ -24,7 +8,6 @@
 #include "input/InputStream.hxx"
 #include "tag/Builder.hxx"
 #include "tag/Table.hxx"
-#include "util/StringView.hxx"
 #include "lib/expat/ExpatParser.hxx"
 
 #include <string.h>
@@ -158,8 +141,7 @@ xspf_end_element(void *user_data, const XML_Char *element_name)
 	case XspfParser::TAG:
 		if (!parser->value.empty())
 			parser->tag_builder.AddItem(parser->tag_type,
-						    StringView(parser->value.data(),
-							       parser->value.length()));
+						    parser->value);
 
 		parser->state = XspfParser::TRACK;
 		break;
@@ -213,12 +195,12 @@ xspf_open_stream(InputStreamPtr &&is)
 	return std::make_unique<MemorySongEnumerator>(std::move(parser.songs));
 }
 
-static const char *const xspf_suffixes[] = {
+static constexpr const char *xspf_suffixes[] = {
 	"xspf",
 	nullptr
 };
 
-static const char *const xspf_mime_types[] = {
+static constexpr const char *xspf_mime_types[] = {
 	"application/xspf+xml",
 	nullptr
 };
