@@ -1,27 +1,9 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "FixString.hxx"
-#include "util/Alloc.hxx"
+#include "util/AllocatedArray.hxx"
 #include "util/CharUtil.hxx"
-#include "util/WritableBuffer.hxx"
-#include "util/StringView.hxx"
 #include "util/UTF8.hxx"
 
 #include <algorithm>
@@ -29,7 +11,7 @@
 
 #include <stdlib.h>
 
-gcc_pure
+[[gnu::pure]]
 static const char *
 FindInvalidUTF8(const char *p, const char *const end) noexcept
 {
@@ -55,15 +37,15 @@ FindInvalidUTF8(const char *p, const char *const end) noexcept
 /**
  * Replace invalid sequences with the question mark.
  */
-static WritableBuffer<char>
-patch_utf8(StringView src, const char *_invalid)
+static AllocatedArray<char>
+patch_utf8(std::string_view src, const char *_invalid) noexcept
 {
 	/* duplicate the string, and replace invalid bytes in that
 	   buffer */
-	char *dest = (char *)xmemdup(src.data, src.size);
-	char *const end = dest + src.size;
+	AllocatedArray<char> dest{src};
+	char *const end = dest.data() + src.size();
 
-	char *invalid = dest + (_invalid - src.data);
+	char *invalid = dest.data() + (_invalid - src.data());
 	do {
 		*invalid = '?';
 
@@ -71,14 +53,14 @@ patch_utf8(StringView src, const char *_invalid)
 		invalid = const_cast<char *>(__invalid);
 	} while (invalid != nullptr);
 
-	return { dest, src.size };
+	return dest;
 }
 
-static WritableBuffer<char>
-fix_utf8(StringView p)
+static AllocatedArray<char>
+fix_utf8(std::string_view p) noexcept
 {
 	/* check if the string is already valid UTF-8 */
-	const char *invalid = FindInvalidUTF8(p.begin(), p.end());
+	const char *invalid = FindInvalidUTF8(p.data(), p.data() + p.size());
 	if (invalid == nullptr)
 		return nullptr;
 
@@ -86,8 +68,9 @@ fix_utf8(StringView p)
 	return patch_utf8(p, invalid);
 }
 
+[[gnu::pure]]
 static const char *
-find_non_printable(StringView p)
+find_non_printable(std::string_view p) noexcept
 {
 	for (const char &ch : p)
 		if (IsNonPrintableASCII(ch))
@@ -100,25 +83,25 @@ find_non_printable(StringView p)
  * Clears all non-printable characters, convert them to space.
  * Returns nullptr if nothing needs to be cleared.
  */
-static WritableBuffer<char>
-clear_non_printable(StringView src)
+static AllocatedArray<char>
+clear_non_printable(std::string_view src)
 {
 	const char *first = find_non_printable(src);
 	if (first == nullptr)
 		return nullptr;
 
-	char *dest = (char *)xmemdup(src.data, src.size);
+	AllocatedArray<char> dest{src};
 
-	for (size_t i = first - src.data; i < src.size; ++i)
+	for (size_t i = first - src.data(); i < src.size(); ++i)
 		if (IsNonPrintableASCII(dest[i]))
 			dest[i] = ' ';
 
-	return { dest, src.size };
+	return dest;
 }
 
-gcc_pure
+[[gnu::pure]]
 static bool
-IsSafe(StringView s) noexcept
+IsSafe(std::string_view s) noexcept
 {
 	return std::all_of(s.begin(), s.end(),
 			   [](char ch){
@@ -126,22 +109,20 @@ IsSafe(StringView s) noexcept
 			   });
 }
 
-WritableBuffer<char>
-FixTagString(StringView p)
+AllocatedArray<char>
+FixTagString(std::string_view p) noexcept
 {
 	if (IsSafe(p))
 		/* optimistic optimization for the common case */
 		return nullptr;
 
 	auto utf8 = fix_utf8(p);
-	if (!utf8.IsNull())
-		p = {utf8.data, utf8.size};
+	if (utf8 != nullptr)
+		p = {utf8.data(), utf8.size()};
 
-	WritableBuffer<char> cleared = clear_non_printable(p);
-	if (cleared.IsNull())
-		cleared = utf8;
-	else
-		free(utf8.data);
+	auto cleared = clear_non_printable(p);
+	if (cleared == nullptr)
+		cleared = std::move(utf8);
 
 	return cleared;
 }
