@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "ShutdownHandler.hxx"
 #include "lib/curl/Global.hxx"
@@ -48,9 +32,9 @@ public:
 				i.first.c_str(), i.second.c_str());
 	}
 
-	void OnData(ConstBuffer<void> data) override {
+	void OnData(std::span<const std::byte> data) override {
 		try {
-			if (fwrite(data.data, data.size, 1, stdout) != 1)
+			if (fwrite(data.data(), data.size(), 1, stdout) != 1)
 				throw std::runtime_error("Failed to write");
 		} catch (...) {
 			OnError(std::current_exception());
